/*
 *  adapted from apcopt.c -- command line options parsing routine for apcupsd.
 *  adapted to pulsard by Philippe Marzouk <philm@users.sourceforge.net>
 * 
 *  Copyright (C) 1999-2000 Riccardo Facchetti <riccardo@master.oasi.gpa.it>
 *
 *  Copyright (C) 1996-99 Andre M. Hedrick <andre@suse.com>
 *  All rights reserved.
 *
 */

/*
 *		       GNU GENERAL PUBLIC LICENSE
 *			  Version 2, June 1991
 *
 *  Copyright (C) 1989, 1991 Free Software Foundation, Inc.
 *			     675 Mass Ave, Cambridge, MA 02139, USA
 *  Everyone is permitted to copy and distribute verbatim copies
 *  of this license document, but changing it is not allowed.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <getopt.h>

#include "pulsard.h"
#include "util.h"

static char *shortoptions = "hVqi:d:a:p:ts:m:";

enum {
	OPT_NOARG,
	OPT_HELP,
	OPT_VERSION,
	OPT_KILLPWR,
	OPT_INTERVAL,
	OPT_DELAY,
	OPT_AUTORESTART,
	OPT_POWERSAVING,
	OPT_PORT,
	OPT_TEST,
	OPT_MAGNITUDE
};

static struct option longoptions[] = {
        { "help",no_argument,NULL,OPT_HELP },
        { "version",no_argument,NULL,OPT_VERSION },
        { "killpower",no_argument,NULL,OPT_KILLPWR },
		{ "check_interval",required_argument,NULL,OPT_INTERVAL },
		{ "delay",required_argument,NULL,OPT_DELAY },
		{ "autorestart",required_argument,NULL,OPT_AUTORESTART },
		{ "powersaving",required_argument,NULL,OPT_POWERSAVING },
		{ "test",no_argument,NULL,OPT_TEST },
		{ "serial-port",required_argument,NULL,OPT_PORT },
		{ "magnitude",required_argument,NULL,OPT_MAGNITUDE },
		{ 0,no_argument,NULL,OPT_NOARG }
};

/*
 * Globals used by command line options.
 */
int show_version = 0;
int kill_ups_power = 0;
int check_interval = 10;
int delay = 20;
int autorestart = 1;
int powersaving = 1;
int test = 0;
char *port = NULL;
int magnitude = 5;

static void print_usage(char *argv[]) {
	printf(
                "usage: %s [options]\n"
                "  Options are as follows:\n"
				"  -s, --serial-port <port>   complete path of the serial port (mandatory)\n"
                "  -q, --killpower            power down UPS after 20 seconds\n"
				"                             or after the delay specified by the delay option\n"
 				"                             without warning!!!\n"
				"  -d, --delay <seconds>      delay before killing power. Default 20\n"
				"  -i, --interval <seconds>   set checking interval to N seconds. Default 10\n"
				"  -a, --autorestart <on|off> autorestart if power is back. Default on\n"
				"  -p, --powersaving <on|off> powersaving mode. Default on\n"
				"  -m, --magnitude <number>   magnitude of change of monitored values (load, voltage, etc.)\n" 
				"                             before reporting to syslog. Default 5\n"
				"  -t, --test                 do not perform signal init nor kill power\n"
                "  -V, --version              display version info\n"
                "  -h, --help                 display this help\n"
                "\n"
                "\n"
                "Copyright (C) 2000 Philippe Marzouk\n",argv[0]);
}

int parse_options(int argc, char *argv[]) {
	int options = 0;
	int oneshot = 0;
	int option_index;
	int errflag = 0;
	int c;

	while(!errflag &&
		   (c = getopt_long(argc, argv, shortoptions,
							longoptions, &option_index)) != -1) {

	options++;

	switch(c) {
		case 'a':
		case OPT_AUTORESTART:
			if (strcmp(optarg,"on\0")==0)
				autorestart = 1;
			else if (strcmp(optarg,"off\0")==0)
				autorestart = 0;
			else {
				printf("You must specify on or off with the -autorestart option\n");
				errflag++;
			}
			break;
		case 'd':
		case OPT_DELAY:
			if(optarg)
				delay=atoi(optarg);
			break;
		case 'i':
		case OPT_INTERVAL:
			if(optarg)
				check_interval=atoi(optarg);
			break;
		case 'm':
		case OPT_MAGNITUDE:
			if (optarg)
				magnitude = atoi(optarg);
			break;
		case 'p':
		case OPT_POWERSAVING:
			if (strcmp(optarg,"on\0")==0)
				powersaving = 1;
			else if (strcmp(optarg,"off\0")==0)
				powersaving = 0;
			else {
				printf("You must specify on or off with the -autorestart option\n");
				errflag++;
			}	
			break;
		case 'q':
		case OPT_KILLPWR:
			kill_ups_power = 1;
			break;
		case 's':
		case OPT_PORT:
			port = optarg;
			break;
		case 't':
		case OPT_TEST:
			test=1;
			break;
		case 'V':
		case OPT_VERSION:
			show_version = 1;
			oneshot = 1;
			break;
		case 'h':
		case OPT_HELP:
		default:
			oneshot = 1;
			errflag++;
		}
	}

	if ((oneshot == 1) && options > 1) {
                printf("\nError: too many arguments.\n\n");
		errflag++;
	}

	if (port == NULL && oneshot != 1 ) {
		printf("\nSerial port must be specified\n\n");
		errflag++;
	}

	if (errflag)
		print_usage(argv);

	return errflag;
}
