/*  pulsard - monitor MGE Pulsar UPS
 * 
 *  Copyright (C) 2000 Philippe Marzouk <philm@users.sourceforge.net>
 *  All rights reserved.
 *
 */

/*
 *		       GNU GENERAL PUBLIC LICENSE
 *			  Version 2, June 1991
 *
 *  Copyright (C) 1989, 1991 Free Software Foundation, Inc.
 *			     675 Mass Ave, Cambridge, MA 02139, USA
 *  Everyone is permitted to copy and distribute verbatim copies
 *  of this license document, but changing it is not allowed.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <gpio/gpio.h>
#include <syslog.h>
#include <string.h>
#include <signal.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <linux/limits.h>

#include "pulsard.h"
#include "serial.h"
#include "util.h"
#include "options.h"

#define MAX_TRY	3

#define STR_INIT "\x5A\x0D\x0A" /* Z : init */
#define LEN_INIT 3
#define STR_AX "\x41\x78\x20\x31\x0D\x0A"       /*Ax 1\r\n */
#define LEN_AX 6
#define STR_SI "\x53\x69\x0D\x0A" /* Si\r\n : UPS identification */
#define LEN_SI 4
#define STR_BN "\x42\x6E\x0D\x0A" /* Bn : battery autonomy */
#define LEN_BN 4
#define STR_BL "\x42\x6C\x0D\x0A" /* Bl : battery charge */
#define LEN_BL 4
#define STR_ID "\x53\x69\x20\x31\x0D\x0A" /* Si 1 : ID string */
#define LEN_ID 6
#define STR_SS "\x53\x73\x0D\x0A"	/* Ss\r\n */
#define LEN_SS 4
#define STR_UV "\x55\x76\x0D\x0A"	/* Uv : incoming voltage */
#define LEN_UV 4
#define STR_UF "\x55\x66\x0D\x0A"   /* Uf : incoming frequency */
#define LEN_UF 4
#define STR_LL "\x4C\x6C\x0D\x0A"	/* Ll : UPS Load */
#define LEN_LL 4
#define STR_PKTEND "\x0D\x0A"		/* \r\n */
#define LEN_PKTEND 2
#define STR_SX "\x53\x78"			/* Sx */
#define LEN_SX 2
#define STR_SN "\x53\x6E"			/* Sn : delay before power off */


char recv_buffer[255];
int power_fail=0, overload=0, low_battery=0;
int input_volt=0, input_freq=0, time_remaining=0;
int load=0,ups_autonomy=0,battery_level=0;
char progname[NAME_MAX]="pulsard\0";
char pidfile[PATH_MAX];
int commstatus=0;

/*
*	terminate the program gracefully
*/
void sigexit(int status)
{
	int pid=0;
	FILE *fp;

	syslog(LOG_INFO,"UPS Daemon terminated");
	if ((fp=fopen(pidfile,"r"))!=NULL) {
		fscanf(fp,"%u",&pid);
		fclose(fp);
	}
	if (pid==getpid()) { /* we only want to remove our pidfile */
		if(unlink(pidfile)==-1)
			syslog(LOG_ALERT,"Could not remove pid file %s",pidfile);
	}
	closelog();
	serial_close();
	exit(status);
}

void reset_values()
{
	power_fail=0;
	overload=0;
	low_battery=0;
	input_volt=0;
	input_freq=0;
	time_remaining=0;
	load=0;
	ups_autonomy=0;
	battery_level=0;
}

main(int argc, char *argv[])
{
	int try,i=0;
	
	FILE *fp;
	char temp[PATH_MAX];

	if (parse_options(argc,argv)>0) 
		exit(-1);

	if (show_version==1) {
		printf("version : 1.0\n");
		exit(1);
	}

	if(getuid()!=0 && test!=1) {
		printf("%s must be run as root!\n",argv[0]);
		exit(-1);
	}

	strcpy(temp,argv[0]);
	if (strrchr(temp,'/') != NULL) {
		i = strrchr(temp,'/') - temp;  
		strcpy(progname,temp+i+1);
	}
	else 
		strcpy(progname,temp);

	if(kill_ups_power==0) {
		sprintf(pidfile,"%s/%s.pid",RUNDIR,progname);
		if ((fp=fopen(pidfile,"r"))!=NULL) {
			printf("pidfile %s exists, program already running, hopes that's ok!\n",pidfile);
			fclose(fp);
		}
	
		if ((fp=fopen(pidfile,"w+"))!=NULL) {
			fprintf(fp,"%u",getpid());
			fclose(fp);
		}
		else
			syslog(LOG_ALERT,"Could not write pid file in %s",RUNDIR);
	}

	setsid();
	signal (SIGTERM, sigexit);
	/* signal (SIGCHLD, SIG_IGN); */

	if(kill_ups_power != 1) {
		openlog(progname,LOG_PID,FACILITY);
		syslog (LOG_INFO, "UPS daemon started");
		syslog (LOG_INFO, "checking interval : %i sec.",check_interval);
	}

	umask(022);

	if(serial_init()!=0) {
		syslog(LOG_CRIT,"Error during init, quitting");
		sigexit(-1);
	}

	while(1) {
		if (commstatus==0) {
		/* ensure that we have the right settings just in case */
			serial_set_settings();
			serial_set_pin();
		/* Start comm with UPS */
			if (ups_start() == 0) {
				reset_values();
		/* config UPS modes */
				for(try=1;try<MAX_TRY;try++) {
					if (set_autorestart()==-2 || set_autorestart()==0)
						break;
					if (try==MAX_TRY) 
						syslog(LOG_ERR,"Could not set autorestart mode");
				}

				for(try=1;try<MAX_TRY;try++) {
					if (set_powersaving()==-2 || set_powersaving()==0)
						break;
					if (try==MAX_TRY) 
						syslog(LOG_ERR,"Could not set powersaving mode");
				}
			
		/* Kill Power if necessary */
				if(kill_ups_power==1) {
					if (test==1) {
						syslog(LOG_DEBUG,"Here I should have killed power, but I'm in test mode");
						printf("Here I should have killed power, but I'm in test mode\n");
					}
					else if(kill_power()==-1) {
						syslog(LOG_ERR,"Error while killing power");
						printf("Error while killing power\n");
					}
					sigexit(-1);
				}
				if (test != 1 && (fp = fopen (PWRFILE, "w+")) != NULL) {
					fprintf(fp,"OK");
					fclose(fp);
				}
				else {
					syslog(LOG_ERR, "Could not write to %s - exiting",PWRFILE);
					sigexit(-1);
				}
			}
		}
		else {
		/* Do the regular testings  */
			ups_status();
			if(power_fail==0)
				power_in_value();
			ups_load();
			autonomy();
			battery_charge();
			sleep(check_interval);
		}
	}

	sigexit(-1);
}



/*
* Initialize communication with UPS
*
* return 0 on success, -1 on failure 
*/
int ups_start()
{
	char buffer[10];
	char *pkt;
	char *id_string;
	int try;
	
	strcpy(buffer,STR_INIT);

	serial_send(buffer,LEN_INIT);
	packet_recv();
	serial_send(buffer,LEN_INIT);
	packet_recv();
	strcpy(buffer,STR_AX);
	serial_send(buffer,LEN_AX);		
	packet_recv();

	if(identify()==0) {
		commstatus=1;
		if (kill_ups_power!=1) {
			for(try=1;try<MAX_TRY;try++) {
				id_string = get_id_string();
				if(id_string) {
					syslog(LOG_INFO,"UPS Model : %s",id_string);
					return 0;
				}
			}
		}
		else
			return 0;
	}
	else
		syslog(LOG_CRIT, "No communication with UPS!");
	
	commstatus=0;
	return -1;
}
	
int identify()
{
	char buffer[10];
	char *pkt;

	strcpy(buffer,STR_SI);
	serial_send(buffer,LEN_SI); 

	pkt = packet_recv();
	if(pkt) {
		if(pkt[0]=='?') return -1;
		if (strncmp(pkt,"3000",4)==0) {
			syslog(LOG_INFO,"Pulsar UPS detected");
			return 0;
		}
		else
			syslog(LOG_INFO,"not a Pulsar UPS!");
	}
	
	return -1;
}


/*
* Get ID String from UPS
*
* 
*/
char *get_id_string()
{
	char buffer[10];

	strcpy(buffer,STR_ID);
    serial_send(buffer,LEN_ID);
	
	return packet_recv();
}

/*
* Turn UPS Off
*
* return -1 on failure
*/
int kill_power()
{
	char buffer[10];
	char *pkt;
	int try,bufsize=0;

	printf("Setting delay\n");
	bufsize=sprintf(buffer,"%s %i\r\n",STR_SN,delay);
	for (try=1;try<MAX_TRY;try++) {
		serial_send(buffer,bufsize);
		pkt=packet_recv();
		if(sizeof(pkt) == 4 && pkt[0] == 'O' && pkt[1] == 'K')
			break;
		else if (try==MAX_TRY) 
			syslog(LOG_INFO,"Could not set delay for power off, using default!");
	}
	printf("turning power off\n");
	bufsize=sprintf(buffer,"%s 0\r\n",STR_SX);
	for (try=1;try<MAX_TRY;try++) {
		serial_send(buffer,bufsize);
		pkt=packet_recv();
		if(sizeof(pkt) == 4 && pkt[0] == 'O' && pkt[1] == 'K') {
			if(!unlink(pidfile))
				printf("could not remove pid file : %s\n",pidfile);
			sigexit(0); 
		}
		else if (try==MAX_TRY) 
			printf("Could not turn power off!");
	}
	return -1;	
}
/*
* Get general status of the UPS
*
* return 0 on success, -1 on failure
*/
int ups_status()
{
	char buffer[10];
	char *pkt;
	FILE *fp;
	pid_t init_pid=1;

	if (commstatus==0)
		return -1;

	strcpy(buffer,STR_SS);
	serial_send(buffer,LEN_SS);

	pkt=packet_recv();	
	if(pkt) {
		if (pkt[4]=='1') {
			if (low_battery==0) {
				low_battery=1;
				syslog(LOG_EMERG,"Battery low! - shutdown immediatly");
				if (test != 1 && (fp = fopen (PWRFILE, "w+")) != NULL) {
					fprintf(fp,"LOW");
					fclose(fp);
				}
				else
					syslog(LOG_ALERT, "Could not write to %s",PWRFILE);
				if (test!=1) 
					kill(init_pid,SIGPWR);
			}
		}
		if (pkt[4]=='0') low_battery=0;
		if (pkt[5]=='1') {
			if (power_fail==0) {
				power_fail=1;
				syslog(LOG_CRIT,"Power Fail!");
				if (test != 1 && (fp = fopen (PWRFILE, "w+")) != NULL) {
					fprintf(fp,"FAIL");
					fclose(fp);
				}
				else
					syslog(LOG_ALERT, "Could not write to %s",PWRFILE);
				
				if (test!=1)
					kill(init_pid,SIGPWR);
			}
		}
		if (pkt[5]=='0') {
			if (power_fail==1) {
				syslog(LOG_CRIT,"Power Restored!");
				if (test != 1) {
					if ((fp = fopen (PWRFILE, "w+")) != NULL) {
						fprintf(fp,"OK");
						fclose(fp);
					}
					else
						syslog(LOG_ALERT, "Could not write to %s",PWRFILE);
				}
				if (test!=1)
					kill(init_pid,SIGPWR);
				power_fail=0;
			}
		}
		if (pkt[6]=='1') {
			if (overload==0) {
				overload=1;
				syslog(LOG_EMERG,"UPS overloaded!");
			}
		}
		if (pkt[6]=='0') overload=0;
		return 0;
	}

	commstatus=0; /* if we are here, no packet was received */
	return -1;
}

/*
* Get the status of the batteries
*
*
*/ 
int battery_status()
{
	char buffer[10];
	char *pkt;

	if (commstatus==0)
		return -1;

	strcpy(buffer,STR_SS);
	serial_send(buffer,LEN_SS);

	pkt=packet_recv();	
	
//	syslog(LOG_DEBUG,"SS: %x %x %x %x %x",pkt[0],pkt[1],pkt[2],pkt[11],pkt[12]);

	return 0;
}

/*
* Get the value of the incoming power
* voltage and frequency
*
*/
int power_in_value()
{
	char buffer[10];
	char *pkt;
	int try;

	if (commstatus==0)
		return -1;

	strcpy(buffer,STR_UV);
	serial_send(buffer,LEN_UV);

	pkt=packet_recv();
	if(pkt) {
		if(pkt[0]=='?') /*	command not supported */
			return -1;
		pkt[3]='\0'; /* we only want the first 3 digits, not the decimal */

		/* take in account only if the difference is more than magnitude */
		if((atoi(pkt) >= (input_volt+magnitude)) || (atoi(pkt) <= (input_volt-magnitude))) { 
			if (input_volt!=atoi(pkt)) { /* to not report every value if magnitude = 0*/
				input_volt=atoi(pkt);
				syslog(LOG_INFO,"Input voltage = %i V",input_volt);
			}
		}	
	}
	
	strcpy(buffer,STR_UF);
    serial_send(buffer,LEN_UF);
	for(try=1;try<MAX_TRY;try++) {
		pkt=packet_recv();
		if(pkt && sizeof(pkt)==4) {
			if(pkt[0]=='?') /*	command not supported */
				return -1;
			input_freq=atoi(pkt);
			if(input_freq != CORRECT_FREQUENCY) {
				syslog(LOG_CRIT,"anormal input frequency = %i Hz",input_freq);
			}
			return 0;
		}
	}
	
	commstatus=0; /* if we are here, no packet was received */
	return -1;
}

/*
* Get the load of the UPS
*
*
*/
int ups_load()
{
	char buffer[10];
	char *pkt;

	if (commstatus==0)
		return -1;

	strcpy(buffer,STR_LL);
	serial_send(buffer,LEN_LL);

	pkt=packet_recv();
	if (pkt) {
		if (pkt[0]=='?') /* feature not supported */
			return -1;

		/* allow a difference bigger than magnitude before reporting load change */
		if ((atoi(pkt)>=(load+magnitude)) || (atoi(pkt)<=(load-magnitude))) {
			if (load != atoi(pkt)) { /* to not report every value if magnitude = 0*/
				load=atoi(pkt);
				syslog(LOG_DEBUG,"Load : %i %%",load);
			}
		}
		return 0;
	}

	commstatus=0; /* if we are here, no packet was received */
	return -1;
}

/*
* Get the battery charge
*
*
*/
int battery_charge()
{
	char buffer[10];
	char *pkt;

	if (commstatus==0)
		return -1;

	strcpy(buffer,STR_BL);
	serial_send(buffer,LEN_BL);

	pkt=packet_recv();
	if(pkt) {
		if (pkt[0]=='?') /* feature not supported */
			return -1;

			/* allow a difference of magnitude before reporting battery charge change */
			if ((atoi(pkt) >= (battery_level+magnitude)) 
					|| (atoi(pkt) <= (battery_level-magnitude))) {
				if (battery_level!=atoi(pkt)) { /* to not report every value if magnitude = 0*/
					battery_level=atoi(pkt);
					syslog(LOG_INFO,"Battery charge : %i %%",battery_level);
				}
			}
		return 0;
	}
	commstatus=0; /* if we are here, no packet was received */
	return -1;
}

/*
* Get the UPS autonomy
*
*
*/
int autonomy()
{
	char buffer[10];
	char *pkt;
	FILE *fp;

	if (commstatus==0)
		return -1;

	strcpy(buffer,STR_BN);
	serial_send(buffer,LEN_BN);

	pkt=packet_recv();
	if(pkt) {
		if (pkt[0]=='?') { /* feature not supported */
			if ((fp = fopen (TTLFILE, "w+")) != NULL) {
				fprintf(fp,"0");
				fclose(fp);
			}
			else
				syslog(LOG_ALERT, "Could not write to %s",TTLFILE);
			return 0;
		}
		/*allow a difference specified by magnitude  before reporting change in autonomy */
		if(((ups_autonomy+magnitude) <= (atoi(pkt)/60)) 
			|| ((ups_autonomy-magnitude) >= (atoi(pkt)/60))) {
			if (ups_autonomy!=atoi(pkt)/60) { /* to not report every value if magnitude = 0*/
				ups_autonomy=atoi(pkt)/60;
				syslog(LOG_INFO,"Autonomy : %i min.",ups_autonomy);
				/* write remaining_time in a file for rc.powerfail purpose */
				if ((fp = fopen (TTLFILE, "w+")) != NULL) {
					fprintf(fp,"%i",ups_autonomy);
					fclose(fp);
				}
				else
					syslog(LOG_ALERT, "Could not write to %s",TTLFILE);
			}
		}					
		return 0;
	}
	commstatus=0; /* if we are here, no packet was received */
	return -1;
}

/*
* select Autorestart mode
*
* return 0 on success, -1 on failure, -2 on unsupported
*/
int set_autorestart()
{
	char buffer[10];
	char *pkt;
	int try,bufsize=0;

	if (commstatus==0)
		return -1;

	if (autorestart==1) 
		bufsize=sprintf(buffer,"%s 5\r\n",STR_SX);
	else if (autorestart==0)
		bufsize=sprintf(buffer,"%s 4\r\n",STR_SX);
	else {
		syslog(LOG_DEBUG,"Value of autorestart undefined");
		return -1;
	}
	
	serial_send(buffer,bufsize);
	pkt=packet_recv();
	if(pkt) {
		if((pkt[0] == 'O') && (pkt[1] == 'K'))
			return 0;
		else if (pkt[0] == '?') {
			syslog(LOG_ERR,"setting autorestart mode not supported");
			return -2;
		}
	}

	commstatus=0; /* if we are here, no packet was received */
	return -1;	
}

/*
* select power saving mode
*
* return 0 on success, -1 on failure, -2 on unsupported
*/
int set_powersaving()
{
	char buffer[10];
	char *pkt;
	int try,bufsize=0;

	if (commstatus==0)
		return -1;

	if (powersaving==1) 
		bufsize=sprintf(buffer,"%s 10\r\n",STR_SX);
	else if (autorestart==0)
		bufsize=sprintf(buffer,"%s 11\r\n",STR_SX);
	else {
		syslog(LOG_DEBUG,"Value of powersaving undefined");
		return -1;
	}
	
	serial_send(buffer,bufsize);
	pkt=packet_recv();
	if (pkt) {
		if((pkt[0] == 'O') && (pkt[1] == 'K'))
			return 0;
		else if (pkt[0] == '?') {
			syslog(LOG_ERR,"setting powersaving mode not supported");
			return -2;
		}
	}

	commstatus=0; /* if we are here, no packet was received */
	return -1;	
}

/*
*	Wait for a complete packet
*
*/
char *packet_recv()
{
	
	int c,i;

	i=0;
    while (c=serial_read()) {
        if ((c == GPIO_ERROR) || (c == GPIO_TIMEOUT)) {
            return NULL;
		}
        if (i==255) {
			syslog(LOG_DEBUG,"Buffer overflow");
			return NULL;
		}
//		syslog(LOG_DEBUG,"reu %i: %x",i,c); 
		recv_buffer[i]=c;
		if(c>1) {
			if ((recv_buffer[i-1]=='\n') && (recv_buffer[i]=='\r')) {
				/* we don't want \r and \n */
				recv_buffer[i-1] = '\0';
				recv_buffer[i] = '\0';	
//				syslog(LOG_DEBUG,"Buffer : %s",recv_buffer); 
	//			dump_hex("recv_buffer",recv_buffer,10);
				return recv_buffer;
			}
		}
		i++;
    }
	return NULL;
}

